import { DisplayPosition } from '../position/display-position';
import { componentMap as formComponentMap } from '@cainiaofe/cn-ui';
import { makeFormItemSchema, uuid } from '@/common/util/util';
import { createFormItemSetters } from '@/lowcode/cn-form-alc/prototype/form-item';
import isPlainObject from 'lodash/isPlainObject';

const CnFormGrid = {
  position: [DisplayPosition.formDialog, DisplayPosition.form],
  title: '组合框',
  componentName: 'CnFormGrid',
  formComponent: formComponentMap.CnFormGrid,
  formItemBeforeHandler: (formItem, config) => {
    const {
      formItemConfig = {},
      formProps = {},
      state,
      _context,
      urlParams,
      formValue,
      formInstance,
      isDesign,
      usedComponentList,
    } = config || {};
    // const { formStyle = {} } = formProps;
    // const { cols, readOnly } = formStyle || {}
    if (formItem) {
      const {
        label,
        name,
        readOnly: formItemReadOnly,
        options = {},
      } = formItemConfig;
      const { config = [], style } = options;
      const { cols } = style || {};
      const componentProps = formItem['x-component-props'];
      if (isPlainObject(componentProps)) {
        componentProps.config = undefined;
        componentProps.hasClear = undefined;
        // componentProps.cols = 1;
        componentProps.style = {
          display: 'flex',
        };
      }

      delete formItem['x-decorator'];
      delete formItem['x-decorator-props'];
      formItem.type = 'void';
      formItem.properties = {};

      if (Array.isArray(config) && config.length > 0) {
        config.forEach((item) => {
          const { label, name, componentName } = item || {};
          const gridFormItem = makeFormItemSchema({
            formItemConfig: item,
            isDesign,
            urlParams,
            formValue,
            state,
            usedComponentList,
            formProps,
            _context,
            formInstance,
          });
          formItem.properties[name] = gridFormItem;
        });
      }
    }
  },
  getDefaultProps: () => {
    return {
      // style:{
      //   cols:2,
      // }
    };
  },
  getFormItemDefaultProps: () => {
    return {
      label: '',
      name: `compose_${uuid()}`,
    };
  },
  getPrototypeList: (position) => {
    return [
      {
        name: 'config',
        title: '表单项配置',
        setter: createFormItemSetters({ position }),
      },
      // {
      //   name:'style',
      //   title:'样式',
      //   display: 'plain',
      //   // collapsed: true,
      //   // extraProps:{
      //   //   defaultCollapsed:true,
      //   // },
      //   setter: {
      //     componentName: 'ObjectSetter',
      //     props: {
      //       config: {
      //         items: [
      //           {
      //             name: 'cols',
      //             title: '一行几列',
      //             display: 'inline',
      //             setter: {
      //               componentName: 'RadioGroupSetter',
      //               props: {
      //                 options: [
      //                   { title: '一列', value: 1 },
      //                   { title: '二列', value: 2 },
      //                   { title: '三列', value: 3 },
      //                 ],
      //               },
      //             },
      //           },
      //         ]
      //       }
      //     }
      //   },
      // },
    ];
  },
};

export default CnFormGrid;
