import { Bundle } from '@ali/visualengine';
import React from 'react';
import Icon from './logo.tsx';
import {
  getArraySetterSnippet,
  getCnSelectSetter, getDataSetterSnippet,
  getMixedSetterSnippet, getNodeIdSetterSnippet,
  getObjectSetterSnippet,
} from '@/common/manager/setter-snippet';
import { getJSExpressionPrototype } from '@/common/manager/common-style';
import { getStatusDataSource } from '@/common/util/prototype-util';
import { createDataSourceSetters } from '@/lowcode/cn-form-alc/prototype/form-item';

const CnStatisticCardProBundle = Bundle.createPrototype({
  title: '指标卡（CnStatisticCardPro）',
  componentName: 'CnStatisticCardPro',
  category: '信息展示',
  icon: Icon,
  docUrl:
    'https://cone.cainiao-inc.com/market/awesome?business=internationality&id=3313',
  isContainer: true,
  canHovering: true,
  canSelecting: true,
  canDragging: true,
  enableCopy: true,
  initialChildren:[{
    componentName: 'CnStatisticLeftSide',
    props:{
      children:[
        {
          componentName:'CnFormat'
        }
      ]
    }
  }],
  snippets: [
    {
      screenshot:
        'https://img.alicdn.com/imgextra/i4/O1CN01SWmTZ124PT13qvuzQ_!!6000000007383-0-tps-720-236.jpg',
      label: 'CnStatisticCardPro',
      schema: {
        componentName: 'CnStatisticCardPro',
        props: {},
      },
    },
  ],
  configure: [
    {
      name: 'isCnStatisticCardPro',
      title: 'isCnStatisticCardPro',
      initialValue: true,
      display: 'none',
    },
    {
      name: '_context',
      title: '上下文',
      initialValue: {
        type: 'JSExpression',
        value: 'this',
      },
      display: 'none',
    },
    {
      name: 'title',
      title: '组件名称',
      display: 'inline',
      initialValue: 'DetailInfo1',
      setter: 'StringSetter',
    },
    ...getDataSetterSnippet(),
    {
      title: '高级配置',
      type: 'group',
      collapsed: true,
      display: 'accordion',
      items: [
        getNodeIdSetterSnippet(),
      ],
    },
  ],
});

const CnStatisticLeftSideBundle = Bundle.createPrototype({
  title: '指标卡左侧栏',
  componentName: 'CnStatisticLeftSide',
  category: '信息展示',
  icon: Icon,
  docUrl:
    'https://cone.cainiao-inc.com/market/awesome?business=internationality&id=3313',
  isContainer: true,
  canHovering: true,
  canSelecting: true,
  canDragging: true,
  enableCopy: true,
  snippets: [
    {
      screenshot:
        'https://img.alicdn.com/imgextra/i4/O1CN01SWmTZ124PT13qvuzQ_!!6000000007383-0-tps-720-236.jpg',
      label: 'CnStatisticLeftSide',
      schema: {
        componentName: 'CnStatisticLeftSide',
        props: {},
      },
    },
  ],
  configure: [
    {
      name: 'isCnStatisticLeftSide',
      title: 'isCnStatisticLeftSide',
      initialValue: true,
      display: 'none',
    },
    {
      name: '_context',
      title: '上下文',
      initialValue: {
        type: 'JSExpression',
        value: 'this',
      },
      display: 'none',
    },
    {
      name: 'title',
      title: '组件名称',
      display: 'inline',
      initialValue: 'DetailInfo1',
      setter: 'StringSetter',
    },
  ],
});

const CnStatisticContentBundle = Bundle.createPrototype({
  title: '指标卡中栏',
  componentName: 'CnStatisticContent',
  category: '信息展示',
  icon: Icon,
  docUrl:
    'https://cone.cainiao-inc.com/market/awesome?business=internationality&id=3313',
  isContainer: true,
  canHovering: true,
  canSelecting: true,
  canDragging: true,
  enableCopy: true,
  snippets: [
    {
      screenshot:
        'https://img.alicdn.com/imgextra/i4/O1CN01SWmTZ124PT13qvuzQ_!!6000000007383-0-tps-720-236.jpg',
      label: 'CnStatisticContent',
      schema: {
        componentName: 'CnStatisticContent',
        props: {},
      },
    },
  ],
  configure: [
    {
      name: 'isCnStatisticContent',
      title: 'isCnStatisticContent',
      initialValue: true,
      display: 'none',
    },
    {
      name: '_context',
      title: '上下文',
      initialValue: {
        type: 'JSExpression',
        value: 'this',
      },
      display: 'none',
    },
    {
      name: 'title',
      title: '组件名称',
      display: 'inline',
      initialValue: 'DetailInfo1',
      setter: 'StringSetter',
    },
  ],
});

const CnStatisticHeaderBundle = Bundle.createPrototype({
  title: '指标卡中栏-头部',
  componentName: 'CnStatisticHeader',
  category: '信息展示',
  icon: Icon,
  docUrl:
    'https://cone.cainiao-inc.com/market/awesome?business=internationality&id=3313',
  isContainer: true,
  canHovering: true,
  canSelecting: true,
  canDragging: true,
  enableCopy: true,
  snippets: [
    {
      screenshot:
        'https://img.alicdn.com/imgextra/i4/O1CN01SWmTZ124PT13qvuzQ_!!6000000007383-0-tps-720-236.jpg',
      label: 'CnStatisticHeader',
      schema: {
        componentName: 'CnStatisticHeader',
        props: {},
      },
    },
  ],
  configure: [
    {
      name: 'isCnStatisticHeader',
      title: 'isCnStatisticHeader',
      initialValue: true,
      display: 'none',
    },
    {
      name: '_context',
      title: '上下文',
      initialValue: {
        type: 'JSExpression',
        value: 'this',
      },
      display: 'none',
    },
    {
      name: 'title',
      title: '组件名称',
      display: 'inline',
      initialValue: 'DetailInfo1',
      setter: 'StringSetter',
    },
  ],
});

const CnStatisticTitleBundle = Bundle.createPrototype({
  title: '指标卡中栏-标题',
  componentName: 'CnStatisticTitle',
  category: '信息展示',
  icon: Icon,
  docUrl:
    'https://cone.cainiao-inc.com/market/awesome?business=internationality&id=3313',
  isContainer: true,
  canHovering: true,
  canSelecting: true,
  canDragging: true,
  enableCopy: true,
  snippets: [
    {
      screenshot:
        'https://img.alicdn.com/imgextra/i4/O1CN01SWmTZ124PT13qvuzQ_!!6000000007383-0-tps-720-236.jpg',
      label: 'CnStatisticTitle',
      schema: {
        componentName: 'CnStatisticTitle',
        props: {},
      },
    },
  ],
  configure: [
    {
      name: 'isCnStatisticTitle',
      title: 'isCnStatisticTitle',
      initialValue: true,
      display: 'none',
    },
    {
      name: '_context',
      title: '上下文',
      initialValue: {
        type: 'JSExpression',
        value: 'this',
      },
      display: 'none',
    },
    {
      name: 'title',
      title: '组件名称',
      display: 'inline',
      initialValue: 'DetailInfo1',
      setter: 'StringSetter',
    },
  ],
});

const CnStatisticActionBundle = Bundle.createPrototype({
  title: '指标卡中栏-按钮',
  componentName: 'CnStatisticAction',
  category: '信息展示',
  icon: Icon,
  docUrl:
    'https://cone.cainiao-inc.com/market/awesome?business=internationality&id=3313',
  isContainer: true,
  canHovering: true,
  canSelecting: true,
  canDragging: true,
  enableCopy: true,
  snippets: [
    {
      screenshot:
        'https://img.alicdn.com/imgextra/i4/O1CN01SWmTZ124PT13qvuzQ_!!6000000007383-0-tps-720-236.jpg',
      label: 'CnStatisticAction',
      schema: {
        componentName: 'CnStatisticAction',
        props: {},
      },
    },
  ],
  configure: [
    {
      name: 'isCnStatisticAction',
      title: 'isCnStatisticAction',
      initialValue: true,
      display: 'none',
    },
    {
      name: '_context',
      title: '上下文',
      initialValue: {
        type: 'JSExpression',
        value: 'this',
      },
      display: 'none',
    },
    {
      name: 'title',
      title: '组件名称',
      display: 'inline',
      initialValue: 'DetailInfo1',
      setter: 'StringSetter',
    },
  ],
});

const CnStatisticValueBundle = Bundle.createPrototype({
  title: '指标卡中栏-主数据',
  componentName: 'CnStatisticValue',
  category: '信息展示',
  icon: Icon,
  docUrl:
    'https://cone.cainiao-inc.com/market/awesome?business=internationality&id=3313',
  isContainer: true,
  canHovering: true,
  canSelecting: true,
  canDragging: true,
  enableCopy: true,
  snippets: [
    {
      screenshot:
        'https://img.alicdn.com/imgextra/i4/O1CN01SWmTZ124PT13qvuzQ_!!6000000007383-0-tps-720-236.jpg',
      label: 'CnStatisticValue',
      schema: {
        componentName: 'CnStatisticValue',
        props: {},
      },
    },
  ],
  configure: [
    {
      name: 'isCnStatisticValue',
      title: 'isCnStatisticValue',
      initialValue: true,
      display: 'none',
    },
    {
      name: '_context',
      title: '上下文',
      initialValue: {
        type: 'JSExpression',
        value: 'this',
      },
      display: 'none',
    },
    {
      name: 'title',
      title: '组件名称',
      display: 'inline',
      initialValue: 'DetailInfo1',
      setter: 'StringSetter',
    },
  ],
});

export default [CnStatisticCardProBundle, CnStatisticLeftSideBundle,CnStatisticContentBundle,CnStatisticHeaderBundle,CnStatisticTitleBundle,CnStatisticActionBundle,CnStatisticValueBundle];
