import { ActionSetter, JsonSetter } from '@ali/visualengine-utils';

export default function () {
  return {
    type: 'composite',
    name: 'remote',
    title: '远程数据接口',
    tip: {
      content: '点击 ? 查看数据格式',
      url: 'https://yuque.antfin.com/qg6d5s/standard/json-standard#ikMOd',
    },
    display: 'accordion',
    disabled() {
      return this.getProps().getPropValue('dataType') === 'datasource';
    },
    items: [
      {
        name: 'url',
        title: '数据接口',
        display: 'block',
        initialValue:
          'https://oneapi.alibaba-inc.com/mock/conemock/toffee/cnTableList.json',
        setter: 'StringSetter',
        // supportVariable: true,
      },
      {
        name: 'failCallback',
        title: '请求失败时的回调方法',
        display: 'block',
        setter: (
          <ActionSetter
            defaultCode={`/**
  * cn-table failCallback
  */
  function failCallback() {
    console.log('failed');
  }`}
            defaultActionName='failCallback'
          />
        ),
      },
      {
        name: 'params',
        title: '额外参数',
        display: 'block',
        initialValue: {},
        setter: <JsonSetter />,
        // supportVariable: true,
      },
      {
        name: 'requestOptions',
        title: 'axios请求配置',
        display: 'block',
        initialValue: {},
        setter: <JsonSetter />,
        // supportVariable: true,
      },
      {
        name: 'responseTransform',
        title: '返回值转换方法',
        display: 'block',
        setter: (
          <ActionSetter
            defaultCode={`/**
  * cn-table responseTransform
  * @param res any 返回数据
  */
  function responseTransform(res) {
    return {
      tableColumns: {},
      tableData: [],
      paging: {},
    }
  }`}
            defaultActionName='responseTransform'
          />
        ),
      },
      {
        name: 'failCallback',
        title: '失败回调方法',
        display: 'block',
        setter: (
          <ActionSetter
            defaultCode={`/**
  * cn-table failCallback
  */
  function failCallback() {
    console.log('failed');
  }`}
            defaultActionName='failCallback'
          />
        ),
      },
    ],
  };
}
